@@ -173,17 +173,25 @@ documented in the doc string of the :mod:`ae.i18n` module.
 A more complex example app demonstrating the features of this context help system can be found in the repository of the
 `kivy lisz demo app <https://gitlab.com/ae-group/kivy_lisz>`_.
 
-app tours (beta - WIP)
+app tours
 =========
 
-:class:`TourBase`
+The following classes are providing a solid fundament to implement tours for your app:
 
-:class:`TourDropdownFromButton`
+* :class:`TourBase`
+* :class:`TourDropdownFromButton`
+* :class:`OnboardingTour`
+* :class:`UserPreferencesTour`
 
-:class:`OnboardingTour`
 
-:class:`UserPreferencesTour`
+UI-specific implementation
+--------------------------
 
+To complete the implementation of the app tours, the UI-specific framework has to provide a tour layout class, which is
+highlighting the widget explained and to display a tooltip and the tour page texts.
+
+Optionally for the 'user_registration' page of the :class:`OnboardingTour` a open user name editor flow has to be
+implemented.
 """
 from abc import abstractmethod, ABC
 from copy import deepcopy
@@ -195,7 +203,7 @@ from ae.gui_app import (
     FLOW_KEY_SEP, flow_action, flow_class_name, flow_path_id, id_of_flow, update_tap_kwargs, MainAppBase)
 
 
-__version__ = '0.1.28'
+__version__ = '0.1.30'
 
 
 register_package_translations()
@@ -677,7 +685,8 @@ class HelpAppBase(MainAppBase, ABC):
 
     def on_app_started(self):
         """ last app init event on app startup. """
-        self.start_app_tour()
+        if self.user_id not in self.registered_users:
+            self.start_app_tour()
 
     def on_flow_popup_close(self, _flow_key: str, _popup_args: Dict[str, Any]) -> bool:
         """ overwritten popup close handler of FlowPopup widget to reset help widget/text. """
@@ -769,6 +778,7 @@ class TourBase:
     def __init__(self, main_app: HelpAppBase):
         main_app.vpo(f"TourBase.__init__(): tour overlay={main_app.tour_layout}")
         self.auto_switch_next_page_request = None
+        self._delayed_setup_layout_call = None
         self._saved_app_states: Dict[str, Any] = dict()
 
         self.main_app = main_app
@@ -777,7 +787,7 @@ class TourBase:
 
         self.page_ids = self.page_ids.copy()        # copy page ids from class to instance attribute
         self.page_idx = 0
-        self.came_back = False
+        self.last_page_idx = None
 
         self.backup_app_states()
 
@@ -792,7 +802,7 @@ class TourBase:
         """ initiate automatic switch to next tour page. """
         main_app = self.main_app
         delay = self.page_data.get('next_page_delay', 9.6)
-        main_app.vpo(f"TourOverlay.auto_switch_next_page from page idx={self.page_idx} delay={delay}")
+        main_app.vpo(f"TourOverlay.auto_switch_next_page from page #{self.page_idx} delay={delay}")
         if self.page_idx + 1 < len(self.page_ids):
             self.auto_switch_next_page_request = main_app.call_method_delayed(delay, self.next_page)
 
@@ -802,6 +812,11 @@ class TourBase:
         main_app.vpo("TourBase.backup_app_states")
         self._saved_app_states = deepcopy(main_app.retrieve_app_states())
 
+    @property
+    def last_page_id(self) -> Optional[str]:
+        """ determine last displayed tour page id. """
+        return None if self.last_page_idx is None else self.page_ids[self.last_page_idx]    # type: ignore # silly mypy
+
     def load_page_data(self):
         """ load page before switching to it (and maybe reload after preparing app flow and before setup of layout). """
         page_idx = self.page_idx
@@ -825,32 +840,29 @@ class TourBase:
 
     def next_page(self):
         """ switch to next tour page. """
-        self.came_back = False
         self.teardown_page()
         self.setup_next_page()
 
         ids = self.page_ids
         assert self.page_idx + 1 < len(ids), f"TourBase.next_page missing {self.__class__.__name__}:{self.page_idx + 1}"
-        this_page_id = ids[self.page_idx]
+        self.last_page_idx = self.page_idx
         self.page_idx += 1
-        next_page_id = ids[self.page_idx]
-        self.main_app.vpo(f"TourBase.next_page(): tour page{self.page_idx} id={this_page_id}->{next_page_id}")
+        self.main_app.vpo(f"TourBase.next_page #{self.page_idx} id={ids[self.last_page_idx]}->{ids[self.page_idx]}")
 
         self.setup_page()
 
     def prev_page(self):
         """ switch to previous tour page. """
-        self.came_back = True
         self.auto_switch_cancel()
         self.teardown_page()
 
         self.setup_prev_page()
 
+        ids = self.page_ids
         assert self.page_idx > 0, f"TourBase.prev_page wrong/missing page {self.__class__.__name__}:{self.page_idx - 1}"
-        this_page_id = self.page_ids[self.page_idx]
+        self.last_page_idx = self.page_idx
         self.page_idx -= 1
-        prev_page_id = self.page_ids[self.page_idx]
-        self.main_app.vpo(f"TourBase.prev_page(): tour page{self.page_idx} id={this_page_id}->{prev_page_id}")
+        self.main_app.vpo(f"TourBase.prev_page #{self.page_idx} id={ids[self.last_page_idx]}->{ids[self.page_idx]}")
 
         self.setup_page()
 
@@ -897,6 +909,7 @@ class TourBase:
 
     def setup_layout(self):
         """ setup/prepare tour overlay/layout after switch of tour page. """
+        self._delayed_setup_layout_call = None
         main_app = self.main_app
         layout = self.layout
 
@@ -923,7 +936,7 @@ class TourBase:
         self.load_page_data()
 
         app_flow_delay = self.page_data.get('app_flow_delay', 1.2)  # > 0.9 to complete auto width animation
-        self.main_app.call_method_delayed(app_flow_delay, self.setup_layout)
+        self._delayed_setup_layout_call = self.main_app.call_method_delayed(app_flow_delay, self.setup_layout)
 
     def setup_prev_page(self):
         """ setup app flow to display the previous tour page. """
@@ -961,12 +974,15 @@ class TourBase:
         self.main_app.vpo("TourBase.start")
         self.reset_app_states()
         self.page_idx = 0
-        self.came_back = False
+        self.last_page_idx = None
         self.setup_page()
 
     def stop(self):
         """ stop/cancel tour. """
         self.main_app.vpo("TourBase.stop")
+        if self._delayed_setup_layout_call:
+            self._delayed_setup_layout_call.cancel()
+            self._delayed_setup_layout_call = None
         self.auto_switch_cancel()
         self.teardown_page()
         self.restore_app_states()
@@ -976,8 +992,13 @@ class TourBase:
         self.main_app.vpo("TourBase.teardown_page")
 
     def update_page_ids(self):
-        """ update/change page ids after app flow setup and before the tour overlay/layout setup. """
-        self.main_app.vpo(f"TourBase.update_page_ids ids={self.page_ids}")
+        """ update/change page ids after app flow setup and before the tour overlay/layout setup.
+
+        override this method to change the page_ids in a running tour. don't forget to reload the page data (by calling
+        :meth:`.load_page_data`) if you remove/replace the current page, or to correct/decrement the attribute values
+        of :attr:`.last_page_idx` and :attr:`.page_idx` if you remove a page which is preceding the current page.
+        """
+        self.main_app.vpo(f"TourBase.update_page_ids {self.page_ids}")
 
 
 class TourDropdownFromButton(TourBase):
@@ -996,23 +1017,23 @@ class TourDropdownFromButton(TourBase):
         """ overwrite to prevent close of dropdown if dropdown child would open another dropdown. """
         super().setup_prev_page()
         if self.page_idx == 1:
-            prev_page_id = self.page_ids[self.page_idx - 1]
-            if flow_action(prev_page_id) == 'open':
+            if flow_action(self.page_ids[self.page_idx - 1]) == 'open':
                 self.main_app.popups_opened()[-1].close()
 
     def update_page_ids(self):
         """ prepare all tour pages - first page is explaining opening dropdown button. """
         super().update_page_ids()
         page_ids = self.page_ids
-        if self.page_idx == 1 and len(page_ids) == 2 and page_ids[1] == TourDropdownFromButton.determine_page_ids:
-            popups = self.main_app.popups_opened()
+        if self.page_idx == 1 and page_ids[-1] == TourDropdownFromButton.determine_page_ids:
+            main_app = self.main_app
+            popups = main_app.popups_opened()
             if not popups:
-                self.main_app.po("TourDropDownFromButton.update_page_ids: dropdown not opened")
+                main_app.po("TourDropDownFromButton.update_page_ids: dropdown not opened")
                 return
 
-            children = self.main_app.widget_tourable_children_page_ids(popups[-1])
+            children = main_app.widget_tourable_children_page_ids(popups[-1])
             if not children:
-                self.main_app.po(f"TourDropDownFromButton.update_page_ids missing tour-able child in {popups[-1]}")
+                main_app.po(f"TourDropDownFromButton.update_page_ids missing tour-able child in {popups[-1]}")
                 return
 
             page_ids.remove(TourDropdownFromButton.determine_page_ids)
@@ -1028,26 +1049,34 @@ _OPEN_USER_PREFERENCES_FLOW_ID = id_of_flow('open', 'user_preferences')
 
 class OnboardingTour(TourBase):
     """ onboarding tour for first app start. """
-    page_ids = ['', 'switch_page', 'help_activate', 'tooltip', 'responsible_layout', 'layout_font_size', 'stop_tour']
+    page_ids = ['', 'page_switching', 'tip_help_intro', 'tip_help_tooltip', 'responsible_layout', 'layout_font_size',
+                'tour_end', 'user_registration']
 
     pages_explained_matcher = dict(
-        help_activate=lambda widget: widget.__class__.__name__ == 'HelpToggler',
-        tooltip=lambda widget: getattr(widget, 'tap_flow_id', None) == _OPEN_USER_PREFERENCES_FLOW_ID,
+        tip_help_intro=lambda widget: widget.__class__.__name__ == 'HelpToggler',
+        tip_help_tooltip=lambda widget: getattr(widget, 'tap_flow_id', None) == _OPEN_USER_PREFERENCES_FLOW_ID,
         layout_font_size=lambda widget: getattr(widget, 'app_state_name', None) == 'font_size',
     )
 
     def setup_page(self):
         """ overridden to open user preferences dropdown in responsible_layout tour page. """
         super().setup_page()
-        if self.page_ids[self.page_idx] == 'layout_font_size':
+
+        page_id = self.page_ids[self.page_idx]
+        if page_id == 'layout_font_size':
             main_app = self.main_app
             flow_id = _OPEN_USER_PREFERENCES_FLOW_ID
             wid = main_app.widget_by_flow_id(flow_id)
             main_app.change_flow(flow_id, **update_tap_kwargs(wid))
 
+        elif page_id == 'user_registration':
+            self.layout.stop_tour()
+            self.main_app.change_flow(id_of_flow('open', 'user_name_editor'))
+
     def teardown_page(self):
         """ overridden to close the opened user preferences dropdown on leaving responsible_layout tour page. """
         super().teardown_page()
+
         if self.page_ids[self.page_idx] == 'layout_font_size':
             popups = self.main_app.popups_opened()
             if popups:
@@ -1056,17 +1085,11 @@ class OnboardingTour(TourBase):
     def update_page_ids(self):
         """ overridden to remove page (only showing once). """
         super().update_page_ids()
-        try:
-            remove_idx = self.page_ids.index('switch_page')
-        except ValueError:
-            return              # already removed this tour page
-
-        delta_idx = self.page_idx - remove_idx
-        if delta_idx == 1 or (delta_idx == -1 and self.came_back):
-            self.page_ids.remove('switch_page')
-            if delta_idx == 1:
+        if self.last_page_id == 'page_switching':
+            self.page_ids.remove('page_switching')
+            if self.last_page_idx == self.page_idx - 1:
+                self.last_page_idx -= 1
                 self.page_idx -= 1
-                self.load_page_data()   # reload data of the page, situated one idx after the removed page
 
 
 class UserPreferencesTour(TourDropdownFromButton):
